from kfsd.apps.core.tests.base_api import BaseAPITestCases
from kfsd.apps.core.utils.attr import AttrUtils


class AttrUtilsTests(BaseAPITestCases):

    def test_format(self):
        data = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_merge_arr_complex_dict1.json')
        attrData = AttrUtils.format(data)
        template0 = attrData.templates[0]
        self.assertEqual(template0.identifier, "cronjob")
        template0Data0 = template0.data[0]
        self.assertEqual(template0Data0.identifier, "1")
        template0Dimensions = template0.dimensions
        self.assertEqual(template0Dimensions, ['addon:volumes', 'addon:serviceaccount'])

    def test_attr_copy(self):
        data = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_merge_arr_complex_dict1.json')
        attr0Data = AttrUtils.format(data)
        attr0template0 = attr0Data.templates[0]
        self.assertEqual(attr0template0.identifier, "cronjob")

        attr1Data = attr0Data.copy()
        attr1template0 = attr1Data.templates[0]
        self.assertEqual(attr1template0.identifier, "cronjob")
        self.assertEqual(attr0Data, attr1Data)



from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent

SECRET_KEY = 'django-insecure-1=9rra#bge4g1rt$lxylq@%n*0ai@sl^qb%*ih(6i)9te24&ne'

DEBUG = True

ALLOWED_HOSTS = []

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'kfsd.apps.frontend',
    'kfsd.apps.core',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',

    'kfsd.apps.core.middleware.config.KubefacetsConfigMiddleware',
    'kfsd.apps.core.middleware.token.KubefacetsTokenMiddleware',
    'kfsd.apps.core.middleware.key.KubefacetsAPIKeyMiddleware',
]

ROOT_URLCONF = 'kfsd.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'kfsd.wsgi.application'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    }
}

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True

STATIC_URL = 'static/'

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

REST_FRAMEWORK = {
    'DEFAULT_SCHEMA_CLASS': 'drf_spectacular.openapi.AutoSchema',
    'KUBEFACETS': {
        "STACKTRACE": False
    },
}

SPECTACULAR_SETTINGS = {
    'TITLE': 'KFSD Utils as a Service',
    'VERSION': '1.0.0',
    "COMPONENT_SPLIT_REQUEST": True,
    "ENUM_ADD_EXPLICIT_BLANK_NULL_CHOICE": False,
    'SERVE_INCLUDE_SCHEMA': False,
    "POSTPROCESSING_HOOKS": [],
    'SERVE_AUTHENTICATION': None,
    "APPEND_COMPONENTS": {
        "securitySchemes": {
            "ApiKeyAuth": {
                "type": "apiKey",
                "in": "header",
                "name": "X-APIKey"
            }
        }
    },
    "SECURITY": [{"ApiKeyAuth": [], }],
}

KUBEFACETS = {
    "config": {
        "is_local_config": True,
        "lookup_dimension_keys": ["env"],
        "local": [
            {
                "setting": ["master"],
                "gateway": {
                    "host": "http://localhost:8002/apis/",
                    "api_key": "9a02f7923aa22e69e0e2858d682a0c227ae0f3ce125a41c61d",
                    "auth": {
                        "token": {
                            "verify_token": "auth/token/verify/"
                        }
                    }
                },
                "certs": {
                    "host": "http://localhost:8002",
                    "jwt_authority_id": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,JWT=Login",
                    "token_gen_uri": "jwt-authorities/{}/token/gen/",
                    "token_publickey_uri": "jwt-authorities/{}/publickey/",
                    "token_decode_uri": "jwt-authorities/{}/token/dec/",
                    "tokens": {
                        "access": {
                            "lifetime_in_mins": 30
                        },
                        "refresh": {
                            "lifetime_in_mins": 300
                        }
                    }
                },
                "auth_fe": {
                    "host": "http://localhost:8000",
                    "login_url": "accounts/signin/",
                    "verify_email_url": "accounts/register/email/",
                    "register_verify_success_url": "http://localhost:8000/accounts/index"
                },
                "auth_api": {
                    "host": "http://localhost:8001",
                    "api_key": "9a02f7923aa22e69e0e2858d682a0c227ae0f3ce125a41c61d",
                    "user_login_url": "login/user/",
                    "user_register_url": "user/",
                    "user_exists_url": "user/{}/",
                    "verify_email_url": "verify/",
                    "verify_tmp_tokens_url": "verify/{}/tokens/",
                    "user_tokens_url": "tokens/{}/login/",
                    "token_extract_url": "tokens/extract/",
                    "access_token_refresh_url": "tokens/access/renew/",
                    "user_identifier_prefix": "USER={}"
                },
                "cookie": {
                    "access": {
                        "key": "access_token",
                        "secure": False,
                        "http_only": True,
                        "same_site": "lax"
                    },
                    "refresh": {
                        "key": "refresh_token",
                        "secure": False,
                        "http_only": True,
                        "same_site": "lax"
                    }
                }
            },
            {
                "setting": ["env:dev"],
                "certs": {
                    "host": "http://localhost:8002/"
                }
            }
        ]
    }
}



from kfsd.apps.core.auth.key import APIKeyUser


class KubefacetsAPIKeyMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request, *args, **kwargs):
        request.key = APIKeyUser(request)
        response = self.get_response(request)
        return response



from drf_spectacular.utils import OpenApiExample


class CommonV1Doc:
    @staticmethod
    def config_examples():
        return [
            OpenApiExample(
                'CONFIG',
                media_type='application/json',
                value={
                    "output": {
                        "value": {
                            "templates": [
                                {
                                    "template": "istio/networking.istio.io/v1alpha3/gateway.json",
                                    "dimensions": [],
                                    "globalvars": {
                                        "__TEMPLATE_ID__": "gateway"
                                    }
                                }
                            ],
                            "overrides": {
                                "gateway": {
                                    "spec": {
                                        "servers": [
                                            {
                                                "port": "{{ values.gateway.virtualhost0.listen }}",
                                                "hosts": "{{ values.gateway.virtualhost0.hosts }}"
                                            }
                                        ]
                                    }
                                }
                            },
                            "values": {
                                "gateway": {
                                    "metadata": {
                                        "name": "istio-gateway",
                                        "namespace": "istio-system"
                                    },
                                    "virtualhost0": {
                                        "listen": {
                                            "number": 80,
                                            "name": "http-snappyguide",
                                            "protocol": "HTTP"
                                        },
                                        "hosts": [
                                            "dev.snappyguide.com",
                                            "dev.accounts.snappyguide.com"
                                        ]
                                    }
                                }
                            }
                        }
                    },
                    "op": "CONFIG"
                },
                request_only=False,
                response_only=False
            ),
            OpenApiExample(
                'CONFIG',
                media_type='application/json',
                value={
                    "op": "CONFIG",
                    "input": {
                        "dimensions": {"environment": "k8s", "cluster": "mac", "type": "dev"},
                        "raw_config": [
                            {
                                "setting": [
                                    "master"
                                ],
                                "templates": [
                                    {
                                        "template": "istio/networking.istio.io/v1alpha3/gateway.json",
                                        "dimensions": [],
                                        "globalvars": {
                                            "__TEMPLATE_ID__": "gateway"
                                        }
                                    }
                                ],
                                "overrides": {
                                    "gateway": {
                                        "spec": {
                                            "servers": [
                                                {
                                                    "port": "{{ values.gateway.virtualhost0.listen }}",
                                                    "hosts": "{{ values.gateway.virtualhost0.hosts }}"
                                                }
                                            ]
                                        }
                                    }
                                },
                                "values": {
                                    "gateway": {
                                        "metadata": {
                                            "name": "istio-gateway",
                                            "namespace": "istio-system"
                                        },
                                        "virtualhost0": {
                                            "listen": {
                                                "number": 80,
                                                "name": "http-snappyguide",
                                                "protocol": "HTTP"
                                            }
                                        }
                                    }
                                }
                            },
                            {
                                "setting": [
                                    "environment:k8s",
                                    "cluster:mac",
                                    "type:dev"
                                ],
                                "values": {
                                    "gateway": {
                                        "virtualhost0": {
                                            "hosts": [
                                                "dev.snappyguide.com",
                                                "dev.accounts.snappyguide.com"
                                            ]
                                        }
                                    }
                                }
                            },
                            {
                                "setting": [
                                    "environment:k8s",
                                    "cluster:inhouse",
                                    "type:prod"
                                ],
                                "values": {
                                    "gateway": {
                                        "virtualhost0": {
                                            "hosts": [
                                                "snappyguide.com",
                                                "accounts.snappyguide.com"
                                            ]
                                        }
                                    }
                                }
                            }
                        ]
                    }
                },
                request_only=True,
                response_only=False
            ),
        ]



from collections.abc import Mapping
import functools

from kfsd.apps.core.common.logger import Logger, LogLevel

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class DictUtils:

    @staticmethod
    def get_all_keys(arr, d):
        for k, v in d.items():
            arr.append(k)
            if isinstance(d.get(k), dict) and isinstance(v, Mapping):
                DictUtils.get_all_keys(arr, d.get(k, {}))

    @staticmethod
    def filter_by_keys_neg(dictionary: dict, visibleKeys: list) -> dict:
        return {k: v for k, v in dictionary.items() if k not in visibleKeys}

    @staticmethod
    def get(d: dict, k: str, default=None):
        if k in d:
            return d[k]
        return default

    @staticmethod
    def merge(**kwargs):
        dict1 = DictUtils.get(kwargs, "dict1", {})
        dict2 = DictUtils.get(kwargs, "dict2", {})
        merge_recursive = DictUtils.get(kwargs, "merge_recursive", True)
        arr_rm_dupes = DictUtils.get(kwargs, "arr_rm_dupes", True)
        arr_dict_lookup_key = DictUtils.get(kwargs, "arr_dict_lookupkey", "identifier")
        is_arr_sorted = DictUtils.get(kwargs, "is_arr_sorted", False)
        if not merge_recursive:
            return {**dict1, **dict2}
        return DictUtils.recursive_merge(dict1, dict2, arr_rm_dupes, arr_dict_lookup_key, is_arr_sorted)

    @staticmethod
    def recursive_merge(dict1, dict2, arr_rm_dupes=True, arr_dict_lookup_key="identifier", is_arr_sorted=True):
        for k, v in dict2.items():
            if isinstance(dict1.get(k), dict) and isinstance(v, Mapping):
                dict1[k] = DictUtils.merge(dict1=dict1.get(k, {}), dict2=v, arr_rm_dupes=arr_rm_dupes, is_arr_sorted=is_arr_sorted)
            elif isinstance(dict1.get(k), list) and isinstance(v, list):
                unionArr = dict1.get(k) + v
                dict1[k] = DictUtils.match_and_merge_arr(unionArr, arr_rm_dupes, arr_dict_lookup_key, is_arr_sorted)
            else:
                dict1[k] = v
        return dict1

    @staticmethod
    def match_and_merge_arr(arr, arr_rm_dupes=True, arr_dict_lookup_key="identifier", is_arr_sorted=True):
        isAllItemsAsDict = DictUtils.confirm_arr_items_as_dict(arr, arr_dict_lookup_key)
        if not isAllItemsAsDict or len(arr) == 0:
            arrItems = list(set(arr)) if arr_rm_dupes else arr
            return sorted(arrItems) if is_arr_sorted else arrItems

        baseDict = DictUtils.convert_arr_to_dict(arr[0], arr_dict_lookup_key)
        for item in arr[1::]:
            baseDict = DictUtils.merge(
                dict1=baseDict,
                dict2=DictUtils.convert_arr_to_dict(item, arr_dict_lookup_key),
                arr_rm_dupes=arr_rm_dupes,
                arr_dict_lookup_key=arr_dict_lookup_key,
                is_arr_sorted=is_arr_sorted
            )

        return [v for k, v in baseDict.items()]

    @staticmethod
    def convert_arr_to_dict(arr, lookupkey="identifier"):
        return {arr[lookupkey]: arr}

    @staticmethod
    def confirm_arr_items_as_dict(arr, arr_dict_lookup_key="identfier"):
        for item in arr:
            if not isinstance(item, dict) or arr_dict_lookup_key not in item:
                return False
        return True

    @staticmethod
    def create_hierarchy(keys, value):
        return functools.reduce((lambda x, y: {y: x}), keys[::-1], value)

    @staticmethod
    def key_exists(d: dict, k: str) -> bool:
        if k in d:
            return True
        return False

    @staticmethod
    def get_by_path(dictionary: dict, path: str):
        keys = path.split(".")
        return functools.reduce(lambda d, key: (d.get(key) if isinstance(d, dict) else None) if d else None, keys, dictionary)

    @staticmethod
    def filter_by_keys(dictionary: dict, visibleKeys: list) -> dict:
        return {k: v for k, v in dictionary.items() if k in visibleKeys}

    @staticmethod
    def key_exists_multi(d: dict, keys: list) -> bool:
        if all(key in d for key in keys):
            return True
        return False

    @staticmethod
    def create(**kwargs):
        return kwargs

    @staticmethod
    def sort_by_values(d, isReverse=False):
        sortedD = sorted(d.items(), key=lambda x: x[1], reverse=isReverse)
        return dict(sortedD)



from kfsd.apps.core.utils.http.headers.cookie import Cookie
from kfsd.apps.core.utils.http.headers.contenttype import ContentType
from kfsd.apps.core.utils.http.headers.apikey import APIKey
from kfsd.apps.core.utils.http.headers.csrf import CSRF


class Headers(Cookie, ContentType, APIKey, CSRF):
    def __init__(self):
        Cookie.__init__(self)
        ContentType.__init__(self)
        APIKey.__init__(self)
        CSRF.__init__(self)
        self.__headers = {}

    def getReqHeaders(self):
        if self.hasCookies():
            self.__headers[self.COOKIE_HEADER_KEY] = self.cookiesToHeaderStr()

        if self.hasContentType():
            self.__headers[self.CONTENTTYPE_HEADER_KEY] = self.getContentType()

        if self.hasAPIKey():
            self.__headers[self.APIKEY_HEADER_KEY] = self.getAPIKey()

        if self.hasCSRF():
            self.__headers[self.CSRF_HEADER_KEY] = self.getCSRF()

        return self.__headers



from kfsd.apps.core.utils.http.django.cookie import Cookie


class DjangoResponse(Cookie):
    def __init__(self, response=None):
        self.__response = response
        Cookie.__init__(self, response)

    def getResponse(self):
        return self.__response

    def setResponse(self, response):
        self.__response = response



from rest_framework import generics
from rest_framework.views import Response


class GetAPIView(generics.GenericAPIView):
    def post(self, request, *args, **kwargs):
        queryset = self.get_queryset()
        serializer = self.get_serializer(queryset)
        return Response(serializer.data)



from kfsd.apps.core.common.logger import Logger, LogLevel
from kfsd.apps.endpoints.serializers.utils.system import SystemOutputRespSerializer
from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.utils.system import System


class SystemHandler:
    OP = "op"
    INPUT = "input"
    CHECKSUM = "CHECKSUM"
    UUID = "UUID"
    SECRET = "SECRET"
    KEY = "KEY"
    ENCRYPT_KEY = "ENCRYPT_KEY"
    OS_ARCH = "OS_ARCH"
    HOST_IP = "HOST_IP"
    NIC = "NIC"
    HOSTNAME = "HOSTNAME"
    OS = "OS"

    def __init__(self, input):
        self.__logger = Logger.getSingleton(__name__, LogLevel.DEBUG)
        self.__input = input
        self.__system = System()

    def methodMappings(self, op):
        mapping = {
            self.CHECKSUM: self.genChecksum,
            self.UUID: self.genUUID,
            self.SECRET: self.genSecret,
            self.KEY: self.genKey,
            self.ENCRYPT_KEY: self.encryptKey,
            self.OS_ARCH: self.osArch,
            self.HOST_IP: self.hostIP,
            self.NIC: self.getNIC,
            self.HOSTNAME: self.getHostName,
            self.OS: self.getOS
        }
        return mapping[op]

    def gen(self):
        op = DictUtils.get(self.__input, self.OP)
        return self.methodMappings(op)()

    def genOutput(self, key, value):
        data = {
            "op": key,
            "output": {
                "value": value
            }
        }
        outputSerializer = SystemOutputRespSerializer(data=data)
        outputSerializer.is_valid()
        return outputSerializer.data

    def genChecksum(self):
        data = DictUtils.get_by_path(self.__input, "input.data")
        return self.genOutput(self.CHECKSUM, System.checksum(data))

    def genUUID(self):
        len = DictUtils.get_by_path(self.__input, "input.len")
        return self.genOutput(self.CHECKSUM, System.uuid(len))

    def genSecret(self):
        len = DictUtils.get_by_path(self.__input, "input.len")
        return self.genOutput(self.CHECKSUM, System.secret(len))

    def genKey(self):
        len = DictUtils.get_by_path(self.__input, "input.len")
        return self.genOutput(self.CHECKSUM, System.api_key(len))

    def encryptKey(self):
        key = DictUtils.get_by_path(self.__input, "input.key")
        return self.genOutput(self.ENCRYPT_KEY, self.__system.encryptKey(key))

    def osArch(self):
        return self.genOutput(self.OS_ARCH, self.__system.osArch())

    def hostIP(self):
        return self.genOutput(self.HOST_IP, self.__system.hostIP())

    def getNIC(self):
        return self.genOutput(self.NIC, self.__system.getNIC())

    def getHostName(self):
        return self.genOutput(self.HOSTNAME, self.__system.getHostName())

    def getOS(self):
        return self.genOutput(self.OS, self.__system.getOS())



"""General API Views"""
from typing import Any
from django.urls import reverse
from django.views.generic import TemplateView


class APIDocsView(TemplateView):
    """Show browser view based on rapi-doc"""

    template_name = "v1/docs/browser.html"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        path = self.request.build_absolute_uri(
            reverse("schema-api",)
        )
        return super().get_context_data(path=path, **kwargs)



from rest_framework.exceptions import ValidationError
from rest_framework.exceptions import APIException
from rest_framework.views import Response, exception_handler
from rest_framework import status
from django.core.exceptions import ValidationError as CoreValidationError
from django.db import IntegrityError
from django.conf import settings
import json
import traceback


class KubefacetsAPIException(APIException):
    default_detail = "Unexpected Error"
    status_code = 500
    default_code = "unexpected_error"

    def __init__(self, detail, defaultCode, statusCode):
        self.detail = detail
        self.status_code = statusCode
        self.default_code = defaultCode


def KubefacetsAPIExceptionHandler(ex, context):
    response = exception_handler(ex, context)
    request = context["request"]
    errorData = {
        "status": "ERROR",
        "path": request.path,
        "method": request.method,
        "content_type": request.content_type,
        "query_params": request.query_params,
        "body": request.data,
        "cookies": request.COOKIES,
        "error": ex.__str__()
    }
    errorJson = json.dumps(errorData, indent=4)
    print(errorJson)

    # print error to console
    if settings.DEBUG:
        print("[[ STACKTRACE ]]")
        traceback.print_exc()

    if isinstance(ex, IntegrityError) and not response:
        response = Response(
            {
                'detail': ex.__str__(),
                'code': "bad_request"
            },
            status=status.HTTP_400_BAD_REQUEST
        )

    if isinstance(ex, TypeError) and not response:
        response = Response(
            {
                'detail': ex.__str__(),
                'code': "system_error"
            },
            status=status.HTTP_500_INTERNAL_SERVER_ERROR
        )

    if isinstance(ex, ValidationError):
        response = Response(
            {
                'detail': ex.args[0],
                'code': "bad_request"
            },
            status=status.HTTP_400_BAD_REQUEST
        )

    if isinstance(ex, CoreValidationError):
        response = Response(
            {
                'detail': ex.args[0],
                'code': "bad_request"
            },
            status=status.HTTP_400_BAD_REQUEST
        )

    if isinstance(ex, KubefacetsAPIException):
        response = Response(
            {
                'detail': ex.detail,
                'code': ex.default_code
            },
            status=ex.status_code
        )
    return response



from drf_spectacular.utils import OpenApiExample


class UtilsV1Doc:

    @staticmethod
    def attr_examples():
        return [
            OpenApiExample(
                'EXPR',
                media_type='application/json',
                value={
                    "op": "EXPR",
                    "input": {
                        "dict": {
                            "id": 1,
                            "age": 41,
                            "bio": {
                                "address": "Bangalore"
                            }
                        },
                        "expr": "request.bio.address"
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'EXPR',
                media_type='application/json',
                value={
                    "output": {
                        "value": "Bangalore"
                    },
                    "op": "EXPR"
                },
                request_only=False,
                response_only=True
            ),
        ]

    @staticmethod
    def arr_examples():
        return [
            OpenApiExample(
                'MERGE (Arr of Dicts)',
                media_type='application/json',
                value={
                    "op": "MERGE",
                    "input": {
                        "lookup_key": "id",
                        "arr1": [
                            {
                                "id": 1,
                                "name": "Gokul"
                            },
                            {
                                "id": 2,
                                "name": "Lavanya"
                            },
                            {
                                "id": 4,
                                "name": "Sana"
                            },
                            {
                                "id": 5,
                                "name": "Cat"
                            }
                        ],
                        "arr2": [
                            {
                                "id": 1,
                                "age": 41
                            },
                            {
                                "id": 2,
                                "age": 36
                            },
                            {
                                "id": 3,
                                "age": 13
                            },
                            {
                                "id": 3,
                                "age": 13
                            },
                            {
                                "id": 5,
                                "name": "Dog"
                            }
                        ],
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'MERGE (Arr of Dicts)',
                media_type='application/json',
                value={
                    "output": {
                        "value": [
                            {
                                "id": 1,
                                "name": "Gokul",
                                "age": 41
                            },
                            {
                                "id": 2,
                                "name": "Lavanya",
                                "age": 36
                            },
                            {
                                "id": 4,
                                "name": "Sana"
                            },
                            {
                                "id": 5,
                                "name": "Dog"
                            },
                            {
                                "id": 3,
                                "age": 13
                            }
                        ]
                    },
                    "op": "MERGE"
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'MERGE',
                media_type='application/json',
                value={
                    "op": "MERGE",
                    "input": {
                        "is_uniq": False,
                        "arr1": ["gokul", "lavanya"],
                        "arr2": ["lavanya", "diya", "sana"],
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'MERGE',
                media_type='application/json',
                value={
                    "op": "MERGE",
                    "output": {
                        "value": ["gokul", "lavanya", "lavanya", "diya", "sana"]
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'MERGE (Uniq)',
                media_type='application/json',
                value={
                    "op": "MERGE",
                    "input": {
                        "is_uniq": True,
                        "arr1": ["gokul", "lavanya"],
                        "arr2": ["lavanya", "diya", "sana"],
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'MERGE (Uniq)',
                media_type='application/json',
                value={
                    "op": "MERGE",
                    "output": {
                        "value": ["gokul", "lavanya", "diya", "sana"]
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'INTERSECTION',
                media_type='application/json',
                value={
                    "op": "INTERSECTION",
                    "input": {
                        "arr1": ["gokul", "lavanya"],
                        "arr2": ["lavanya", "diya", "sana"],
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'INTERSECTION',
                media_type='application/json',
                value={
                    "op": "INTERSECTION",
                    "output": {
                        "value": ["lavanya"]
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'JOIN',
                media_type='application/json',
                value={
                    "op": "JOIN",
                    "input": {
                        "arr": ["gokul", "nathan"],
                        "str": ","
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'JOIN',
                media_type='application/json',
                value={
                    "op": "JOIN",
                    "output": {
                        "value": "gokul,nathan"
                    }
                },
                request_only=False,
                response_only=True
            )
        ]

    @staticmethod
    def system_examples():
        return [
            OpenApiExample(
                'CHECKSUM ( Str )',
                media_type='application/json',
                value={
                    "op": "CHECKSUM",
                    "input": {
                        "data": "gokul"
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'CHECKSUM ( Dict )',
                media_type='application/json',
                value={
                    "op": "CHECKSUM",
                    "input": {
                        "data": {
                            "key": "value"
                        }
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'UUID',
                media_type='application/json',
                value={
                    "op": "UUID",
                    "input": {
                        "len": 32
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'SECRET',
                media_type='application/json',
                value={
                    "op": "SECRET",
                    "input": {
                        "len": 32
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'KEY',
                media_type='application/json',
                value={
                    "op": "KEY",
                    "input": {
                        "len": 10
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'ENCRYPT_KEY',
                media_type='application/json',
                value={
                    "op": "ENCRYPT_KEY",
                    "input": {
                        "key": "gokul"
                    }
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'OS_ARCH',
                media_type='application/json',
                value={
                    "op": "OS_ARCH"
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'HOST_IP',
                media_type='application/json',
                value={
                    "op": "HOST_IP"
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'NIC',
                media_type='application/json',
                value={
                    "op": "NIC"
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'HOSTNAME',
                media_type='application/json',
                value={
                    "op": "HOSTNAME"
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'OS',
                media_type='application/json',
                value={
                    "op": "OS"
                },
                request_only=True,
                response_only=False
            ),
            OpenApiExample(
                'CHECKSUM',
                media_type='application/json',
                value={
                    "op": "CHECKSUM",
                    "output": {
                        "value": 106627619
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'UUID',
                media_type='application/json',
                value={
                    "op": "UUID",
                    "output": {
                        "value": "ad2c18417cecc9b167c9b19ad52dfdc1"
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'SECRET',
                media_type='application/json',
                value={
                    "op": "SECRET",
                    "output": {
                        "value": "P9gOT1OkxSFx2EXUL-fEr6qLoPB8awJuMQiaMI9ObEE"
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'KEY',
                media_type='application/json',
                value={
                    "op": "KEY",
                    "output": {
                        "value": "46e2f9bf60"
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'ENCRYPT_KEY',
                media_type='application/json',
                value={
                    "op": "ENCRYPT_KEY",
                    "output": {
                        "value": "Z29rdWwK"
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'HOST_IP',
                media_type='application/json',
                value={
                    "op": "HOST_IP",
                    "output": {
                        "value": "192.168.2.10"
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'NIC',
                media_type='application/json',
                value={
                    "op": "NIC",
                    "output": {
                        "value": "en0"
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'HOSTNAME',
                media_type='application/json',
                value={
                    "op": "HOSTNAME",
                    "output": {
                        "value": "localhost"
                    }
                },
                request_only=False,
                response_only=True
            ),
            OpenApiExample(
                'OS',
                media_type='application/json',
                value={
                    "op": "OS",
                    "output": {
                        "value": "Darwin"
                    }
                },
                request_only=False,
                response_only=True
            ),
        ]



from kfsd.apps.core.tests.base_api import BaseAPITestCases
from kfsd.apps.core.utils.file import FileUtils
from kfsd.apps.core.exceptions.exec import ExecException


class FileUtilsTests(BaseAPITestCases):

    def test_read_as_json(self):
        json = FileUtils.read_as_json('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys.json')
        self.assertEqual(json["type"], "bio")

    def test_read_as_json_exception(self):
        with self.assertRaises(ExecException):
            FileUtils.read_as_json('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys_tmp.json')

    def test_construct_path(self):
        filePath = FileUtils.construct_path("kfsd/apps/core/tests/v1/data/requests/utils/dict", "test_dict_get_all_keys.json")
        self.assertEqual(filePath, "kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys.json")

    def test_copy_dir(self):
        destDir = "/tmp/utils"
        FileUtils.rm_dir(destDir)
        isCreated = FileUtils.copy_dir("kfsd/apps/core/tests/v1/data/requests/utils", destDir)
        self.assertTrue(isCreated)
        self.assertTrue(FileUtils.path_exists("{}/dict".format(destDir)))
        self.assertTrue(FileUtils.path_exists("{}/dict/test_dict_get_all_keys.json".format(destDir)))

    def test_copy_dir_exception(self):
        with self.assertRaises(ExecException):
            FileUtils.copy_dir("kfsd/apps/core/tests/v1/data/requests/utils", "/var/test")

    def test_create_dir(self):
        destDir = "/tmp/utils"
        FileUtils.rm_dir(destDir)
        self.assertFalse(FileUtils.path_exists(destDir))
        isDirCreated = FileUtils.create_dir(destDir)
        self.assertTrue(isDirCreated)
        self.assertTrue(FileUtils.path_exists(destDir))
        FileUtils.rm_dir(destDir)

        dirDepthPath = destDir+"/folder1/folder2"
        isDirCreated = FileUtils.create_dir(dirDepthPath)
        self.assertTrue(isDirCreated)
        isDirCreated = FileUtils.create_dir(dirDepthPath)
        self.assertFalse(isDirCreated)
        FileUtils.rm_dir(dirDepthPath)

    def test_create_dir_exception(self):
        with self.assertRaises(ExecException):
            destDir = "/var/test/path1"
            FileUtils.create_dir(destDir)

    def test_read_as_text_rm_linebreaks(self):
        textContent = FileUtils.read_as_text('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys.json')
        self.assertTrue(type(textContent) == str)
        self.assertTrue("\n" not in textContent)
        self.assertTrue("Gokul Nathan" in textContent)

    def test_read_as_text_exception(self):
        with self.assertRaises(ExecException):
            FileUtils.read_as_text('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys_tmp.json')

    def test_read_as_text_no_rm_linebreaks(self):
        textContent = FileUtils.read_as_text('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys.json', False)
        self.assertTrue(type(textContent) == str)
        self.assertTrue("\n" in textContent)
        self.assertTrue("Gokul Nathan" in textContent)

    def test_read(self):
        textContent = FileUtils.read('kfsd/apps/core/tests/v1/data/requests/utils/file/test_read.txt')
        self.assertTrue(len(textContent) == 4)
        self.assertTrue(textContent[0] == "My name is Nathan")
        self.assertTrue("\n" not in textContent[0])

        limitedLines = FileUtils.read('kfsd/apps/core/tests/v1/data/requests/utils/file/test_read.txt', 1, 2)
        self.assertTrue(limitedLines[0] == "I live in Bowmanville")

        limitedLines1 = FileUtils.read('kfsd/apps/core/tests/v1/data/requests/utils/file/test_read.txt', 1)
        self.assertTrue(len(limitedLines1) == 3)

        limitedLines2 = FileUtils.read('kfsd/apps/core/tests/v1/data/requests/utils/file/test_read.txt', 1, 2, False)
        self.assertTrue(limitedLines2[0] == "I live in Bowmanville\n")

    def test_write_exception(self):
        with self.assertRaises(ExecException):
            FileUtils.write("/var/tmp.txt", "my content")

    def test_write(self):
        destDir = "/tmp/utils/tmp.txt"
        writeContent = "my content"
        FileUtils.rm_dir(destDir)
        FileUtils.write(destDir, writeContent)
        pathExists = FileUtils.path_exists(destDir)
        self.assertTrue(pathExists)
        readContent = FileUtils.read_as_text(destDir)
        self.assertEqual(writeContent, readContent)

    def test_write_as_json_exception(self):
        destDir = "/var/tmp.json"
        data = {"name": "Gokul"}
        with self.assertRaises(ExecException):
            FileUtils.write_as_json(destDir, data)

    def test_write_as_json(self):
        destDir = "/tmp/utils/folder1/tmp.json"
        data = {"name": "Gokul"}
        FileUtils.write_as_json(destDir, data)
        json_data = FileUtils.read_as_json(destDir)
        self.assertEqual(data, json_data)

    def test_move_file_exception(self):
        with self.assertRaises(ExecException):
            FileUtils.move_file('kfsd/apps/core/tests/v1/data/requests/utils/file/test_read.txt', '/var/test_read.txt')

    def test_move_file(self):
        destDir = "/tmp/utils/folder1/tmp.json"
        data = {"name": "Gokul"}
        FileUtils.write_as_json(destDir, data)
        FileUtils.create_dir("/tmp/utils/folder2")
        moveFilePath = "/tmp/utils/folder2/tmp.json"
        FileUtils.move_file(destDir, moveFilePath)
        pathExists = FileUtils.path_exists(moveFilePath)
        self.assertTrue(pathExists)

    def test_copy_file_exception(self):
        with self.assertRaises(ExecException):
            FileUtils.copy_file('kfsd/apps/core/tests/v1/data/requests/utils/file/test_read.txt', '/var/test_read.txt')

    def test_copy_file(self):
        copiedFile = FileUtils.copy_file('kfsd/apps/core/tests/v1/data/requests/utils/file/test_read.txt', '/tmp/utils/test_read.txt')
        self.assertTrue(copiedFile)

    def test_rm_dir_exception(self):
        with self.assertRaises(ExecException):
            FileUtils.rm_dir("/var")

    def test_rm_dir(self):
        folderPath = "/tmp/utils/folder2"
        FileUtils.create_dir(folderPath)
        self.assertTrue(FileUtils.path_exists(folderPath))

        FileUtils.rm_dir(folderPath)
        self.assertFalse(FileUtils.path_exists(folderPath))

    def test_rm_file(self):
        destFile = "/tmp/utils/folder1/tmp.json"
        data = {"name": "Gokul"}
        FileUtils.write_as_json(destFile, data)
        fileRemoved = FileUtils.rm_file(destFile)
        self.assertTrue(fileRemoved)



import requests
from rest_framework import status

from kfsd.apps.core.exceptions.api import KubefacetsAPIException
from kfsd.apps.core.utils.http.response import Response
from kfsd.apps.core.utils.http.headers.base import Headers


class Request(Headers, Response):
    def __init__(self):
        self.__expStatusCode = None

    def setExpStatusCode(self, code):
        self.__expStatusCode = code

    def request(self, method, url, expStatus, **kwargs):
        try:
            print("Req Headers: {}".format(self.getReqHeaders()))
            resp = method(url, **kwargs)
            self.setResponse(resp)
            if self.isRespValid(expStatus):
                return resp
        except requests.exceptions.Timeout:
            raise KubefacetsAPIException(
                "The server took too long to respond to your request. Please try again later.",
                "server_timed_out",
                status.HTTP_408_REQUEST_TIMEOUT,
            )
        except requests.exceptions.ConnectionError:
            raise KubefacetsAPIException(
                "Service temporarily unavailable. Please try again later.",
                "service_unavailable",
                status.HTTP_503_SERVICE_UNAVAILABLE,
            )

    def post(self, url, expStatus, **kwargs):
        return self.request(requests.post, url, expStatus, **kwargs)

    def get(self, url, expStatus, **kwargs):
        return self.request(requests.get, url, expStatus, **kwargs)

    def delete(self, url, expStatus, **kwargs):
        return self.request(requests.delete, url, expStatus, **kwargs)




class AttrUtils(dict):
    """A dictionary with attribute-style access. It maps attribute access to
    the real dictionary.  """

    def __init__(self, *args, **kwargs):
        dict.__init__(self, *args, **kwargs)

    @staticmethod
    def format_list(value):
        return [AttrUtils.format_by_type(item) for item in value]

    @staticmethod
    def format(value):
        return AttrUtils.format_by_type(value)

    @staticmethod
    def format_by_type(value):
        if isinstance(value, dict):
            return AttrUtils(AttrUtils.format_dict(value))
        elif isinstance(value, list):
            return AttrUtils.format_list(value)
        elif isinstance(value, str):
            return value

        return value

    @staticmethod
    def format_dict(d):
        return AttrUtils({k: AttrUtils.format(v) for (k, v) in d.items()})

    def __getstate__(self):
        return self.__dict__.items()

    def __setstate__(self, items):
        for key, val in items:
            self.__dict__[key] = val

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, dict.__repr__(self))

    def __setitem__(self, key, value):
        return super(AttrUtils, self).__setitem__(key, value)

    def __getitem__(self, name):
        return super(AttrUtils, self).__getitem__(name)

    def __delitem__(self, name):
        return super(AttrUtils, self).__delitem__(name)

    __getattr__ = __getitem__
    __setattr__ = __setitem__

    def copy(self):
        return AttrUtils(self)



from django.test import TestCase
from django.urls import reverse
from unittest.mock import patch


class APIDocsViewTests(TestCase):
    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    def test_get(self, mocked):
        mocked.side_effect = [{}]
        url = reverse("api_doc")
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)



from http import HTTPStatus
from kfsd.apps.core.utils.http.headers.contenttype import ContentType
from kfsd.apps.core.auth.api.gateway import APIGateway
from kfsd.apps.core.utils.http.headers.base import Headers


class TokenAuth(Headers, APIGateway):
    def __init__(self, request=None):
        Headers.__init__(self)
        APIGateway.__init__(self, request)

    def setAPIGatewayHeaders(self):
        self.setCSRF(
            self.getDjangoRequest().getDjangoReqCookies().getCookie("csrftoken")
        )
        self.setAPIKey(self.getApplicationAPIKey())
        self.setContentType(ContentType.APPLICATION_JSON)

    def getTokenUserInfo(self):
        verifyTokenUri = self.getDjangoRequest().findConfigs(
            ["gateway.auth.token.verify_token"]
        )[0]
        verifyTokenCompleteUrl = self.constructUrl(verifyTokenUri)
        self.setAPIGatewayHeaders()
        self.post(
            verifyTokenCompleteUrl,
            HTTPStatus.OK,
            json={
                "cookies": self.getDjangoRequest().getDjangoReqCookies().getAllCookies()
            },
            headers=self.getReqHeaders(),
        )
        return self.getResponse().json()



from rest_framework import status
from django.urls import reverse
from unittest.mock import patch

from kfsd.apps.endpoints.tests.endpoints_test_handler import EndpointsTestHandler


class ConfigurationTestCases(EndpointsTestHandler):
    def setUp(self):
        super().setUp()

    def postYaml(self, url, data, expStatus,):
        response = self.client.post(url, data, format='yaml')
        self.assertEqual(response.status_code, expStatus)
        print(response)

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    def test_common_config_dev(self, mocked):
        postData = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/requests/common/config/test_common_config_dev.json')
        obsResponse = self.post(reverse("common-config"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/responses/common/config/test_common_config_dev.json')
        self.assertEqual(obsResponse, expResponse)

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    def test_common_config_prod(self, mocked):
        postData = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/requests/common/config/test_common_config_prod.json')
        obsResponse = self.post(reverse("common-config"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/responses/common/config/test_common_config_prod.json')
        self.assertEqual(obsResponse, expResponse)



from kfsd.apps.core.auth.base import BaseUser
from kfsd.apps.core.auth.api.key import APIKeyAuth


class APIKeyUser(BaseUser, APIKeyAuth):
    def __init__(self, request):
        # self.setRequest(request)
        self.setUserInfo(self.getApiKeyUserInfo())



"""
Provides TEXT rendering support.
"""
from __future__ import unicode_literals

from rest_framework.renderers import BaseRenderer


class KubefacetsTEXTRenderer(BaseRenderer):
    """
    Renderer which serializes to TEXT.
    """

    media_type = "plain/text"
    format = "text"
    charset = "utf-8"

    def render(self, data, accepted_media_type=None, renderer_context=None):

        return data



from kfsd.apps.core.auth.base import BaseUser
from kfsd.apps.core.auth.api.token import TokenAuth
from kfsd.apps.core.utils.dict import DictUtils


class TokenUser(BaseUser, TokenAuth):
    def __init__(self, request):
        BaseUser.__init__(self)
        TokenAuth.__init__(self, request=request)
        self.setUserInfo(self.getTokenUserInfo())

    def getUserCookies(self):
        userInfo = self.getUserInfo()
        return DictUtils.get_by_path(userInfo, "data.cookies")



from kfsd.apps.core.common.logger import Logger, LogLevel

from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.utils.arr import ArrUtils
from kfsd.apps.endpoints.serializers.utils.arr import ArrUtilsOutputRespSerializer


class ArrHandler:
    OP = "op"
    JOIN = "JOIN"
    INTERSECTION = "INTERSECTION"
    MERGE = "MERGE"

    def __init__(self, input):
        self.__logger = Logger.getSingleton(__name__, LogLevel.DEBUG)
        self.__input = input

    def methodMappings(self, op):
        mapping = {
            self.JOIN: self.join,
            self.INTERSECTION: self.intersection,
            self.MERGE: self.merge,
        }
        return mapping[op]

    def gen(self):
        op = DictUtils.get(self.__input, self.OP)
        return self.methodMappings(op)()

    def genOutput(self, key, value):
        data = {
            "op": key,
            "output": {
                "value": value
            }
        }
        outputSerializer = ArrUtilsOutputRespSerializer(data=data)
        outputSerializer.is_valid()
        return outputSerializer.data

    def join(self):
        arr = DictUtils.get_by_path(self.__input, "input.arr")
        join_str = DictUtils.get_by_path(self.__input, "input.str")
        return self.genOutput(self.JOIN, ArrUtils.join(arr, join_str))

    def intersection(self):
        arr1 = DictUtils.get_by_path(self.__input, "input.arr1")
        arr2 = DictUtils.get_by_path(self.__input, "input.arr2")
        return self.genOutput(self.INTERSECTION, ArrUtils.intersection(arr1, arr2))

    def merge(self):
        arr1 = DictUtils.get_by_path(self.__input, "input.arr1")
        arr2 = DictUtils.get_by_path(self.__input, "input.arr2")
        is_uniq = DictUtils.get_by_path(self.__input, "input.is_uniq")
        arr_lookup_key = DictUtils.get_by_path(self.__input, "input.lookup_key")
        return self.genOutput(self.MERGE, ArrUtils.merge(arr1, arr2, is_uniq, arr_lookup_key if arr_lookup_key else "identifier"))



from kfsd.apps.core.common.logger import Logger, LogLevel

from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.common.configuration import Configuration
from kfsd.apps.endpoints.serializers.common.configuration import ConfigurationOutputRespSerializer


class ConfigurationHandler:
    OP = "op"
    CONFIG = "CONFIG"

    def __init__(self, input):
        self.__logger = Logger.getSingleton(__name__, LogLevel.DEBUG)
        self.__input = input

    def methodMappings(self, op):
        mapping = {
            self.CONFIG: self.genConfig
        }
        return mapping[op]

    def gen(self):
        op = DictUtils.get(self.__input, self.OP)
        return self.methodMappings(op)()

    def genOutput(self, key, value):
        data = {
            "op": key,
            "output": {
                "value": value
            }
        }
        outputSerializer = ConfigurationOutputRespSerializer(data=data)
        outputSerializer.is_valid()
        return outputSerializer.data

    def genConfig(self):
        input = DictUtils.get(self.__input, "input", {})
        rawConfig = DictUtils.get(input, "raw_config", [])
        dimensions = DictUtils.get(input, "dimensions", {})
        recursiveMerge = DictUtils.get(input, "recursive_merge", True)
        arrRmDupes = DictUtils.get(input, "arr_rm_dupes", True)

        config = Configuration(
            settings=rawConfig,
            dimensions=dimensions,
            merge_recursive=recursiveMerge,
            arr_rm_dupes=arrRmDupes
        )
        return self.genOutput(self.CONFIG, config.getFinalConfig())




from rest_framework import serializers


class BaseInputReqSerializer(serializers.Serializer):
    input = serializers.JSONField(default=dict)


class BaseOutputRespSerializer(serializers.Serializer):
    output = serializers.JSONField()


class ErrorSerializer(serializers.Serializer):
    detail = serializers.CharField()
    code = serializers.CharField()


class NotFoundSerializer(serializers.Serializer):
    detail = serializers.CharField()
    code = serializers.CharField()


class SuccessSerializer(serializers.Serializer):
    detail = serializers.CharField()
    code = serializers.CharField()



class ContentType:
    APPLICATION_JSON = "application/json"
    CONTENTTYPE_HEADER_KEY = "Content-Type"

    def __init__(self):
        self.__contentType = None

    def setContentType(self, contentType):
        self.__contentType = contentType

    def getContentType(self):
        return self.__contentType

    def hasContentType(self):
        if self.__contentType:
            return True
        return False



from django.urls import path, include
from drf_spectacular.views import SpectacularAPIView
from rest_framework import routers
from kfsd.apps.endpoints.views.utils.utils import UtilsViewSet
from kfsd.apps.endpoints.views.utils.common import CommonViewSet

router = routers.DefaultRouter()
router.include_format_suffixes = False

router.register("utils", UtilsViewSet, basename="utils")
router.register("common", CommonViewSet, basename="common")

urlpatterns = [
    path('', include(router.urls)),
    path('schema/', SpectacularAPIView.as_view(), name='schema-api'),
]



from setuptools import setup, find_packages

setup(
    name='kfsd',
    version='0.0.11',
    description='Sample App',
    long_description='Sample Pkg',
    long_description_content_type="text/markdown",
    author='Gokul Nathan',
    author_email='nathangokul111@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Django==4.1.6',
        'asgiref==3.6.0',
        'coverage==7.1.0',
        'djangorestframework==3.13.1',
        'flake8==6.0.0',
        'mccabe==0.7.0',
        'netifaces==0.11.0',
        'pycodestyle==2.10.0',
        'pyflakes==3.0.1',
        'pytz==2022.7.1',
        'PyYAML==6.0',
        'sqlparse==0.4.3',
        'drf-spectacular==0.22.1',
        'ruamel.yaml==0.17.21',
        'requests==2.28.2'
    ],
)



from django.conf import settings
from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.utils.system import System
from kfsd.apps.core.common.configuration import Configuration


class KubefacetsConfigMiddleware:

    def __init__(self, get_response):
        self.get_response = get_response
        self.__config = self.genConfig()

    def genConfig(self):
        return self.deriveConfig()

    def deriveConfig(self):
        kubefacetsSettings = settings.KUBEFACETS
        isLocalConfig = DictUtils.get_by_path(kubefacetsSettings, "config.is_local_config")
        lookupDimensions = DictUtils.get_by_path(kubefacetsSettings, "config.lookup_dimension_keys")
        if isLocalConfig:
            localConfig = DictUtils.get_by_path(kubefacetsSettings, "config.local")
            return self.genLocalConfig(lookupDimensions, localConfig)
        return None

    def genLocalConfig(self, dimensionKeys, config):
        dimensions = self.constructDimensionsFromEnv(dimensionKeys)
        return Configuration(settings=config, dimensions=dimensions)

    def constructDimensionsFromEnv(self, dimensionKeys):
        return {key: System.getEnv(key) for key in dimensionKeys}

    def __call__(self, request):
        request.config = self.__config
        response = self.get_response(request)
        return response

    def process_view(self, request, view_func, view_args, view_kwargs):
        return None



from kfsd.apps.core.tests.base_api import BaseAPITestCases
from kfsd.apps.core.common.configuration import Configuration
from kfsd.apps.core.common.yaml import Yaml


class YamlTests(BaseAPITestCases):
    def test_configuration_yaml_dev(self):
        data = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/common/configuration/gateway.json')
        dimensions = {"environment": "k8s", "cluster": "mac", "type": "dev"}
        configObj = Configuration(settings=data, dimensions=dimensions, merge_recursive=True, arr_rm_dupes=True)
        obsConfig = configObj.getFinalConfig()
        yamlObj = Yaml()
        obsData = yamlObj.getYaml(yamlObj.formatDictConfig(obsConfig))
        expData = self.readText('kfsd/apps/core/tests/v1/data/responses/common/yaml/test_configuration_yaml_dev.yaml', False)
        self.assertEqual(expData, obsData)



from kfsd.apps.core.utils.http.base import HTTP
from kfsd.apps.core.utils.http.django.request import DjangoRequest


class APIGateway(HTTP):
    def __init__(self, request=None):
        self.__request = DjangoRequest(request)
        HTTP.__init__(self)

    def getGatewayServiceAPIKey(self):
        return self.getDjangoRequest().findConfigs(["services.api_key"])[0]

    def getDjangoRequest(self):
        return self.__request
